package sshoneypot

import (
	"errors"
	"fmt"
	"github.com/bytedream/sshoneypot/sshoneypot/info"
	"golang.org/x/crypto/ssh"
)


type payloadWindowChange struct {
	Width, Height           uint32

	// always 0
	PixelWidth, PixelHeight uint32
}

type payloadPty struct {
	Term string
	Width, Height           uint32

	// always 0
	PixelWidth, PixelHeight uint32

	Modes []byte
}


func handleRequest(requests <-chan *ssh.Request, term *info.Termina) {
	for request := range requests {
		switch request.Type {
		case "pty-req":
			var pty payloadPty
			ssh.Unmarshal(request.Payload, &pty)

			term.SetSize(int(pty.Width), int(pty.Height))
		case "window-change":
			var windowChange payloadWindowChange
			ssh.Unmarshal(request.Payload, &windowChange)

			term.SetSize(int(windowChange.Width), int(windowChange.Height))
		}
		// sends a reply if wanted
		if request.WantReply {
			request.Reply(true, nil)
		}
	}
}

func parseCommand(command string) ([]string, error) {
	var args []string
	state := "start"
	current := ""
	quote := "\""
	escapeNext := true
	for i := 0; i < len(command); i++ {
		c := command[i]

		if state == "quotes" {
			if string(c) != quote {
				current += string(c)
			} else {
				args = append(args, current)
				current = ""
				state = "start"
			}
			continue
		}

		if escapeNext {
			current += string(c)
			escapeNext = false
			continue
		}

		switch c {
		case '\\':
			escapeNext = true
			continue
		case '"', '\'':
			state = "quotes"
			quote = string(c)
			continue
		}

		if state == "arg" {
			if c == ' ' || c == '\t' {
				args = append(args, current)
				current = ""
				state = "start"
			} else {
				current += string(c)
			}
			continue
		}

		if c != ' ' && c != '\t' {
			state = "arg"
			current += string(c)
		}
	}

	if state == "quotes" {
		return []string{}, errors.New(fmt.Sprintf("Unclosed quote in command line: %s", command))
	}

	if current != "" {
		args = append(args, current)
	}

	return args, nil
}
