package info

type Handler interface {
	//AutoCompleteOptions returns the extra arguments for a command (e.g. the `-a` or `-R` option in the `ls` command)
	AutoCompleteOptions() []string

	//AutoCompleteRequest gets called if the client request auto complete (e.g. if double pressing the `tab` key)
	AutoCompleteRequest(info *Info) (accepted bool)

	//Handle will execute / handle the command the client has entered
	Handle(info *Info)

	//Kill gets called if the user hard stops the command (ctrl + c)
	Kill()
}

type SimpleHandler struct {
	Handler
}

func (sh SimpleHandler) AutoCompleteOptions() []string {
	return make([]string, 0)
}
func (sh SimpleHandler) AutoCompleteRequest(info *Info) (accepted bool) {
	return false
}
func (sh SimpleHandler) Handle(info *Info) {}
func (sh SimpleHandler) Kill(){}
