package info

import (
	"fmt"
	"golang.org/x/crypto/ssh"
	"golang.org/x/term"
	"net"
)

type Termina struct {
	*term.Terminal

	// The current prompt string
	Prompt string

	Width, Height uint32
}

func (t *Termina) SetPrompt(prompt string) {
	t.Prompt = prompt

	t.Terminal.SetPrompt(prompt)
}

type Info struct {
	Conn net.Conn
	Terminal *Termina

	FS *FileSystem
	User *ssh.ServerConn

	FullArgs []string
	Command string
	Args []string

	Stdout bool
	Stderr bool
}

func (info Info) Read() (content string, err error) {
	if info.Terminal != nil {
		oldPrompt := info.Terminal.Prompt
		info.Terminal.SetPrompt("")
		content, err = info.Terminal.ReadLine()
		info.Terminal.SetPrompt(oldPrompt)
	} else {
		var buffer []byte
		_, err = info.Conn.Read(buffer)
		content = string(buffer)
	}
	return content, err
}

func (info Info) ReadPrompt() (content string, err error) {
	if info.Terminal != nil {
		content, err = info.Terminal.ReadLine()
	} else {
		var buffer []byte
		_, err = info.Conn.Read(buffer)
		content = string(buffer)
	}
	return content, err
}

func (info Info) Write(content string) (err error) {
	if info.Stdout {
		if info.Terminal != nil {
			_, err = info.Terminal.Write([]byte(content))
		} else {
			_, err = info.Conn.Write([]byte(content))
		}
	}
	return err
}

func (info Info) Writef(format string, args... interface{}) error {
	return info.Write(fmt.Sprintf(format, args...))
}

func (info Info) Writeln(content string) (err error) {
	return info.Write(content + "\n")
}

func (info Info) Writelnf(format string, args... interface{}) (err error) {
	return info.Writeln(fmt.Sprintf(format, args...))
}

func (info Info) Error(content string) (err error) {
	if info.Stderr {
		if info.Terminal != nil {
			_, err = info.Terminal.Write([]byte(content))
		} else {
			_, err = info.Conn.Write([]byte(content))
		}
	}
	return err
}

func (info Info) Errorf(format string, args... interface{}) error {
	return info.Error(fmt.Sprintf(format, args...))
}

func (info Info) Errorln(content string) (err error) {
	return info.Error(content + "\n")
}

func (info Info) Errorlnf(format string, args... interface{}) (err error) {
	return info.Errorln(fmt.Sprintf(format, args...))
}
