package sshoneypot

import (
	"crypto/rand"
	"crypto/rsa"
	"crypto/x509"
	"encoding/pem"
	"golang.org/x/crypto/ssh"
	"io/ioutil"
)

// LoadSSHKey loads a ssh key out of bytes
func LoadSSHKey(keyBytes []byte) (ssh.Signer, error) {
	// parse the read key bytes
	key, err := ssh.ParsePrivateKey(keyBytes)
	if err != nil {
		return nil, err
	}
	return key, nil
}

// LoadSSHKeyFromFile loads a ssh key out of a file
func LoadSSHKeyFromFile(file string) (ssh.Signer, error) {
	keyBytes, err := ioutil.ReadFile(file)
	if err != nil {
		return nil, err
	}

	return LoadSSHKey(keyBytes)
}

// GenerateSSHKey generates a new rsa ssh key
func GenerateSSHKey() ([]byte, error) {
	// create a key
	privateKey, err := rsa.GenerateKey(rand.Reader, 2048)
	if err != nil {
		return nil, err
	}
	privateKeyBlock := pem.Block{
		Type: "RSA PRIVATE KEY",
		Headers: nil,
		Bytes: x509.MarshalPKCS1PrivateKey(privateKey),
	}
	return pem.EncodeToMemory(&privateKeyBlock), nil
}
